#
#
#

""" Define PT file for domains based on domain defining residues """
__author__ = "Magdalena A. Jonikas"
__version__ = "1.0"
__date__ = " 7/21/09 "

import sys
import simtk.nast.molecule as molecule
import simtk.nast.simplePdb as pdb

class Intermediate(object):
    def __init__(self, xtalIn, domainDefIn):
        self.m=pdb.SimplePdb(pdbFilename=xtalIn)
        self.offset=self.m.data[0]['resNum']
        self.cutoff=1.0
        self.weakforce=1.0
        self.strongforce=5.0
        self.rootname=xtalIn[:-4]
        self.domainGroups={}

    def parseDomains(self, domainFilename):
        dlines = open(domainFilename).readlines()
        domainIndex=0
        domainList=[]
        self.domains={}
        for line in dlines:
            if line[0]=='#':
                if len(domainList)>0:
                    self.domains[domainName]=domainList
                cols = line.split()
                domainName=cols[1]
                domainList = []
            else:
                res = int(line)
                domainList.append(res)
        if len(domainList)>0:
            self.domains[domainName]=domainList
        print "there are %i domains" % len(self.domains.keys())
        print self.domains.keys()

    def calcDist(self, resA, resB):
        xA = self.m.data[resA]['x']
        yA = self.m.data[resA]['y']
        zA = self.m.data[resA]['z']
        xB = self.m.data[resB]['x']
        yB = self.m.data[resB]['y']
        zB = self.m.data[resB]['z']
        dist = ((xA-xB)**2 + (yA-yB)**2 + (zA-zB)**2)**0.5
        return dist

    def defineDomain(self, domainName):
        reslist = self.domains[domainName]
        nearbyResList=[]
        for res in reslist:
            resID = res-self.offset
            for i in range(len(self.m.data)):
                dist=self.calcDist(resID, i)
                if dist<self.cutoff:
                    nearbyResList.append(self.m.data[i]['resNum'])
        self.domainGroups[domainName]=nearbyResList

    def printDomainGroups(self):
        assignedResidues = []
        outFile = open('%s-domainGroups.txt' % self.rootname, 'w')
        for domain in self.domains.keys():
            reslist = self.domainGroups[domain]
            defReslist = self.domains[domain]
            outFile.write('# %s\n' % domain)
            for res in reslist:
                if res in assignedResidues: continue
                if res in defReslist: outFile.write('D %i\n' % res)
                else: outFile.write('%i\n' % res)
                assignedResidues.append(res)
        outFile.close()

    def writeXtalPT(self):
        ptOutXtalWeak = open('%s-F-PT-W.txt' % (self.rootname), 'w')
        ptOutXtalStrong = open('%s-F-PT-S.txt' % (self.rootname), 'w')
        for i in range(len(self.m.data)):
            for j in range(i+1,len(self.m.data)):
                dist=self.calcDist(i, j)
                ptOutXtalWeak.write('%i %i %.2f %.2f\n' % (i+self.offset, j+self.offset, dist, self.weakforce))
                ptOutXtalStrong.write('%i %i %.2f %.2f\n' % (i+self.offset, j+self.offset, dist, self.strongforce))
        ptOutXtalWeak.close()
        ptOutXtalStrong.close()


    def writePT(self, domainName):
        reslist = self.domainGroups[domainName]
        ptOutWeak = open('%s-%s-PT-W.txt' % (self.rootname, domainName), 'w')
        ptOutStrong = open('%s-%s-PT-S.txt' % (self.rootname, domainName), 'w')
        for i in range(len(reslist)):
            resA=reslist[i]
            resi=resA-self.offset
            for j in range(i+1,len(reslist)):
                resB=reslist[j]
                if resB==resA: continue
                resj=resB-self.offset
                dist=self.calcDist(resi,resj)
                if dist>15.0: continue
                ptOutWeak.write('%i %i %.2f %.2f\n' % (resA, resB, dist, self.weakforce))
                ptOutStrong.write('%i %i %.2f %.2f\n' % (resA, resB, dist, self.strongforce))
        ptOutWeak.close()
        ptOutStrong.close()




if __name__=='__main__':
    xtalIn = sys.argv[1]
    domainDefIn = sys.argv[2]

    i=Intermediate(xtalIn, domainDefIn)
    i.parseDomains(domainDefIn)
    i.rootname=xtalIn[0:4]
    for domain in i.domains.keys():
            i.defineDomain(domain)
            print "domain %s \n  %i defining residues \n  %i group residues \n  within %.2f Angstroms" % (domain, len(i.domains[domain]), len(i.domainGroups[domain]), i.cutoff*10.0)
            i.writePT(domain)
    i.printDomainGroups()
    i.writeXtalPT()
